using Random
using UUIDs

rng = Random.MersenneTwister(0)

abstract type Node end

struct DataTaskNode <: Node
   task::AbstractDataTask
   
   # use vectors as sets have way too much memory overhead
   parents::Vector{Node}
   children::Vector{Node}

   # need a unique identifier unique to every *constructed* node
   id::Base.UUID
end

# same as DataTaskNode
struct ComputeTaskNode <: Node
   task::AbstractComputeTask
   parents::Vector{Node}
   children::Vector{Node}
   id::Base.UUID
end

function DataTaskNode(t::AbstractDataTask)
   return DataTaskNode(t, Vector{Node}(), Vector{Node}(), UUIDs.uuid1(rng))
end

function ComputeTaskNode(t::AbstractComputeTask)
   return ComputeTaskNode(t, Vector{Node}(), Vector{Node}(), UUIDs.uuid1(rng))
end

struct Edge
   # edge points from child to parent
   edge::Union{Tuple{DataTaskNode, ComputeTaskNode}, Tuple{ComputeTaskNode, DataTaskNode}}
end
